#include <bits/stdc++.h>

using namespace std;

typedef vector<pair<int, int>> lazy_segment_tree;

void add(lazy_segment_tree& tree, int k, int val) {
    k += tree.size() / 2;
    tree[k].first += val;
    for(k /= 2; k >= 1; k /= 2) tree[k].first = tree[2*k].first + tree[2*k+1].first;
}

void segment_tree(const vector<int>& array, lazy_segment_tree& tree) {
    int size = 1;
    while(size < array.size()) size *= 2;
    tree.resize(size * 2);
    for(int i = 0; i < array.size(); i++) add(tree, i, array[i]);
}

void lazy_propagation(lazy_segment_tree& tree, int node, int length) {
    if(tree[node].second == 0) return;
    tree[2*node].second += tree[node].second;
    tree[2*node + 1].second += tree[node].second;
    tree[node].first += tree[node].second * length;
    tree[node].second = 0;
}

int add_util(lazy_segment_tree& tree, int a, int b, int val, int node, int from, int to) {
    if(from > to || from > b || to < a) return 0;     // out of range

    // [from,to] is contained into [a,b]: update all elements (lazily)
    if(a <= from && to <= b) {
        tree[node].second += val;
        return val * (to - from + 1);
    }
    
    lazy_propagation(tree, node, to - from + 1);
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    int res = add_util(tree, a, b, val, 2*node, from, mid) + add_util(tree, a, b, val, 2*node + 1, mid + 1, to);
    tree[node].first += res;
    return res;
}

void add(lazy_segment_tree& tree, int a, int b, int val) {
    add_util(tree, a, b, val, 1, 0, tree.size()/2 - 1);
}

int sumq_util(lazy_segment_tree& tree, int a, int b, int node, int from, int to) {
    if(from > to || from > b || to < a) return 0;     // out of range

    // [from,to] is contained into [a,b]
    if(a <= from && to <= b) return tree[node].first + tree[node].second * (to - from + 1);
    
    lazy_propagation(tree, node, to - from + 1);
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    return sumq_util(tree, a, b, 2*node, from, mid) + sumq_util(tree, a, b, 2*node + 1, mid + 1, to);
}

int sumq(lazy_segment_tree& tree, int a, int b) {
    return sumq_util(tree, a, b, 1, 0, tree.size()/2 - 1);
}

int main() {
    vector<int> array = {5, 8, 6, 3, 2, 7, 2, 6, 7, 1, 7, 5, 6, 2, 3, 2};
    
    lazy_segment_tree tree;
    segment_tree(array, tree);
    
    cout << sumq(tree, 5, 13) << endl;
    add(tree, 5, 13, 2);
    cout << sumq(tree, 10, 13) << endl;
    
    return 0;
}
